package com.yunzhi.liyuanhui.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.liyuanhui.common.BaseController;
import com.yunzhi.liyuanhui.common.ResponseBean;
import com.yunzhi.liyuanhui.common.StringUtils;
import com.yunzhi.liyuanhui.entity.TaActivityVote;
import com.yunzhi.liyuanhui.entity.TaPerson;
import com.yunzhi.liyuanhui.enums.StatusEnum;
import com.yunzhi.liyuanhui.service.ITaActivityEnrollService;
import com.yunzhi.liyuanhui.service.ITaActivityVoteService;
import org.bouncycastle.cms.PasswordRecipient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.liyuanhui.service.ITaActivityService;
import com.yunzhi.liyuanhui.entity.TaActivity;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    * 活动 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-08
 */
@RestController
@RequestMapping("/")
public class TaActivityController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaActivityController.class);

    @Autowired
    public ITaActivityService iTaActivityService;

    @Autowired
    private ITaActivityVoteService iTaActivityVoteService;

    @Autowired
    private ITaActivityEnrollService iTaActivityEnrollService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/activity",method= RequestMethod.GET)
    public ResponseBean taActivityList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @RequestParam(value ="typeId", required = false) Integer typeId,
                                       @RequestParam(value ="name", required = false) String name,
                                       @RequestParam(value ="status", required = false) Integer status) throws Exception{

        IPage<TaActivity> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaActivity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(null != typeId, "type_id", typeId);
        queryWrapper.like(!StringUtils.isEmpty(name), "name", name);
        queryWrapper.eq(null != status, "status", status);
        queryWrapper.gt(null == status, "status", StatusEnum.DELETED.getCode());
        queryWrapper.orderByDesc("weight, create_date");

        IPage<TaActivity> result = iTaActivityService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/wx/activity",method= RequestMethod.GET)
    public ResponseBean getWxList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                  @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                  @RequestParam(value ="typeId", required = false) Integer typeId,
                                  @RequestParam(value ="toIndex", defaultValue = "false") Boolean toIndex) throws Exception{

        IPage<TaActivity> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaActivity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(null != typeId, "type_id", typeId);
        queryWrapper.eq(toIndex, "to_index", toIndex);
        queryWrapper.gt("status", StatusEnum.READY.getCode());
        queryWrapper.orderByDesc("weight, create_date");

        IPage<TaActivity> result = iTaActivityService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @GetMapping("/wx/person/{personId}/activity")
    public ResponseBean getActivityOfPerson(@PathVariable Integer personId,
                                            @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                            @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        IPage<TaActivity> pg = new Page<>(pageNum, pageSize);
        IPage<TaActivity> result = iTaActivityService.getListByPersonId(pg, personId);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taActivity 实体对象
     * @return
     */
    @RequestMapping(value="/admin/activity",method= RequestMethod.POST)
    public ResponseBean taActivityAdd(@RequestBody TaActivity taActivity) throws Exception{

        if (iTaActivityService.save(taActivity)){
            return ResponseBean.success(iTaActivityService.getById(taActivity.getActivityId()));
        }else {
            return ResponseBean.error("保存失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/admin/activity/{id}", method= RequestMethod.DELETE)
    public ResponseBean taActivityDelete(@PathVariable Integer id) throws Exception{

        UpdateWrapper<TaActivity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusEnum.DELETED.getCode());
        updateWrapper.eq("activity_id", id);

        if(iTaActivityService.update(updateWrapper)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taActivity 实体对象
     * @return
     */
    @RequestMapping(value="/admin/activity/{id}",method= RequestMethod.PUT)
    public ResponseBean taActivityUpdate(@PathVariable Integer id,
                                        @RequestBody TaActivity taActivity) throws Exception{

        if (iTaActivityService.updateById(taActivity)){
            return ResponseBean.success(taActivity);
        }else {
            return ResponseBean.error("更新失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/activity/{id}",method= RequestMethod.GET)
    public ResponseBean taActivityGet(@PathVariable String client,
                                      @PathVariable Integer id,
                                      HttpServletRequest request) throws Exception{
        TaActivity result = iTaActivityService.getActivityProfile(id);

        if (null != result) {
            if (StatusEnum.DELETED.getCode().equals(result.getStatus())) {
                return ResponseBean.error("当前内容已经被删除", ResponseBean.ERROR_ILLEGAL_PARAMS);
            }
        }

        if ("wx".equals(client)) {
            if (StatusEnum.READY.getCode().equals(result.getStatus())) {
                return ResponseBean.error("当前内容不存在或者未发布", ResponseBean.ERROR_ILLEGAL_PARAMS);
            }

            TaPerson taPerson = getPerson(request);
            // 是否参与投票
            if (null != result.getIsVote() && result.getIsVote()) {
                boolean isVoted = iTaActivityVoteService.isVotedBy(result.getActivityId(), taPerson.getPersonId());
                TaActivityVote taActivityVote = iTaActivityVoteService.getBy(result.getActivityId(), taPerson);
                if (null != taActivityVote) {
                    result.setSelfVoted(taActivityVote.getItemId());
                }
            }
            // 是否参与报名
            if (null != result.getIsEnroll() && result.getIsEnroll()) {
                boolean isSelfEnrolled = iTaActivityEnrollService.isEnrolledBy(result.getActivityId(), taPerson);
                result.setIsSelfEnrolled(isSelfEnrolled);
            }

            // 增加 pv 忽略错误
            try {
                result.setPv(result.getPv() + 1);
                iTaActivityService.updateById(result);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return ResponseBean.success(result);
    }
}
