package com.yunzhi.liyuanhui.controller;

import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.liyuanhui.common.*;
import com.yunzhi.liyuanhui.enums.StatusEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.liyuanhui.service.ITaPersonService;
import com.yunzhi.liyuanhui.entity.TaPerson;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 用户 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-08
 */
@RestController
@RequestMapping("/")
public class TaPersonController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaPersonController.class);

    @Autowired
    public ITaPersonService iTaPersonService;

    @Autowired
    WxUtils wxUtils;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/person",method= RequestMethod.GET)
    public ResponseBean taPersonList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                     @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @RequestParam(value ="phone", required = false) String phone,
                                     @RequestParam(value ="name", required = false) String name
                                     ) throws Exception{

        IPage<TaPerson> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(phone), "phone", phone);
        queryWrapper.nested(!StringUtils.isEmpty(name), q -> q.like("name", name).or().like("nickname", name));
        queryWrapper.like(!StringUtils.isEmpty(name), "name", name);
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.nested(q -> q.and(qa -> qa.isNotNull("nickname").ne("nickname", ""))
                .or(qb -> qb.isNotNull("phone").ne("phone", "")));
        queryWrapper.orderByDesc("create_date");

        IPage<TaPerson> result = iTaPersonService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @GetMapping("/admin/person/export")
    public void exportPerson(@RequestParam(value ="phone", required = false) String phone,
                             @RequestParam(value ="name", required = false) String name,
                             HttpServletResponse response) throws Exception {
         List<TaPerson> personList = iTaPersonService.getListBy(phone, name);
         for (TaPerson person: personList) {
             // 设置性别
             if (Integer.valueOf(1) == person.getSex()) {
                 person.setGender("男");
             } else if (Integer.valueOf(1) == person.getSex()) {
                 person.setGender("女");
             } else {
                 person.setGender("未知");
             }
             // 转换成 excel 可识别的时间
             person.setExcelDate(DateUtils.local2Date(person.getCreateDate()));
         }

        ExcelUtils.flush(response, TaPerson.class, personList, "会员信息统计");
    }

    /**
     * 保存对象
     * @param taPerson 实体对象
     * @return
     */
    @RequestMapping(value="/taPerson",method= RequestMethod.POST)
    public ResponseBean taPersonAdd(@RequestBody TaPerson taPerson) throws Exception{

        if (iTaPersonService.save(taPerson)){
            return ResponseBean.success(taPerson);
        }else {
            return ResponseBean.error("保存失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taPerson/{id}", method= RequestMethod.DELETE)
    public ResponseBean taPersonDelete(@PathVariable Integer id) throws Exception{

        if(iTaPersonService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taPerson 实体对象
     * @return
     */
    @RequestMapping(value="/wx/person/{id}",method= RequestMethod.POST)
    public ResponseBean taPersonUpdate(@PathVariable Integer id,
                                        @RequestBody TaPerson taPerson) throws Exception{
        if (null == taPerson.getPersonId()) {
            taPerson.setPersonId(id);
        }

        if (iTaPersonService.updateById(taPerson)){
            return ResponseBean.success(taPerson);
        }else {
            return ResponseBean.error("修改失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/wx/person/{id}",method= RequestMethod.GET)
    public ResponseBean taPersonGet(@PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaPersonService.getById(id));
    }

    /**
     * 获取并更新客户手机
     * @param personId
     * @param params
     * @return
     * @throws Exception
     */
    @PostMapping("/wx/person/{personId}/phone")
    public ResponseBean updatePhone(@PathVariable Integer personId, @RequestBody Map<String, String> params) throws Exception{
        if (null == params) {
            return ResponseBean.error("未获取足够参数", ResponseBean.ERROR_MISSING_PARAMS);
        }

        String sessionKey = params.get("sessionKey");
        String encryptedData = params.get("encryptedData");
        String iv = params.get("iv");

        if (StringUtils.isEmpty(sessionKey) || StringUtils.isEmpty(encryptedData) || StringUtils.isEmpty(iv)) {
            return ResponseBean.error("未获取足够参数", ResponseBean.ERROR_MISSING_PARAMS);
        }

        WxMaPhoneNumberInfo phoneNoInfo = wxUtils.getService().getUserService().getPhoneNoInfo(sessionKey, encryptedData, iv);
        String phone = phoneNoInfo.getPurePhoneNumber();
        if (StringUtils.isEmpty(phone)) {
            return ResponseBean.error("获取手机号失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaPerson taPerson = iTaPersonService.updatePersonPhone(personId, phone);
        return ResponseBean.success(taPerson);
    }
}
