package com.yunzhi.liyuanhui.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.liyuanhui.common.*;
import com.yunzhi.liyuanhui.entity.RecommendExport;
import com.yunzhi.liyuanhui.entity.TaPerson;
import com.yunzhi.liyuanhui.enums.StatusEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.liyuanhui.service.ITaRecommenderService;
import com.yunzhi.liyuanhui.entity.TaRecommender;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 推荐表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-08
 */
@RestController
@RequestMapping("/")
public class TaRecommenderController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaRecommenderController.class);

    @Autowired
    public ITaRecommenderService iTaRecommenderService;


    /**
     * 获取推荐客户列表
     * @param pageNum
     * @param pageSize
     * @param startDate 格式 YYYY-MM-DD
     * @param endDate
     * @param recName 推荐人
     * @param recPhone
     * @param name 被推荐人
     * @param phone
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/admin/recommender",method= RequestMethod.GET)
    public ResponseBean taRecommenderList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                          @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                          @RequestParam(value ="startDate", required = false) String startDate,
                                          @RequestParam(value ="endDate", required = false) String endDate,
                                          @RequestParam(value ="recName", required = false) String recName,
                                          @RequestParam(value ="recPhone", required = false) String recPhone,
                                          @RequestParam(value ="name", required = false) String name,
                                          @RequestParam(value ="phone", required = false) String phone,
                                          @RequestParam(value ="status", required = false) Integer status
                                          ) throws Exception{

        IPage<Map<String, Object>> pg = new Page<>(pageNum, pageSize);

        if (!StringUtils.isEmpty(startDate)) {
            startDate += " 00:00:00";
        }
        if (!StringUtils.isEmpty(endDate)) {
            endDate += " 23:59:59";
        }

        IPage<Map<String, Object>> result = iTaRecommenderService.getListBy(pg, startDate, endDate, recName, recPhone, name, phone, status);
        return ResponseBean.success(result);
    }

    @GetMapping("/admin/recommender/export")
    public void taRecommenderList(@RequestParam(value ="startDate", required = false) String startDate,
                                          @RequestParam(value ="endDate", required = false) String endDate,
                                          @RequestParam(value ="recName", required = false) String recName,
                                          @RequestParam(value ="recPhone", required = false) String recPhone,
                                          @RequestParam(value ="name", required = false) String name,
                                          @RequestParam(value ="phone", required = false) String phone,
                                          @RequestParam(value ="status", required = false) Integer status,
                                          HttpServletResponse response) throws Exception{
        if (!StringUtils.isEmpty(startDate)) {
            startDate += " 00:00:00";
        }
        if (!StringUtils.isEmpty(endDate)) {
            endDate += " 23:59:59";
        }

        List<RecommendExport> result = iTaRecommenderService.getListBy(startDate, endDate, recName, recPhone, name, phone, status);
        for (RecommendExport item: result) {
            // 转换性别
            if ("1".equals(item.getSex())) {
                item.setSex("男");
            } else if ("2".equals(item.getSex())) {
                item.setSex("女");
            } else {
                item.setSex("未知");
            }

            // 转换审核状态
            if ("1".equals(item.getStatus())) {
                item.setStatus("审核通过");
            } else if ("2".equals(item.getStatus())) {
                item.setStatus("审核未通过");
            } else {
                item.setStatus("未审核");
            }
        }

        ExcelUtils.flush(response, RecommendExport.class, result, StringUtils.strToUnicode("推荐用户"));
    }

    @GetMapping("/wx/person/{personId}/recommender")
    public ResponseBean getRecommenderOf(@PathVariable Integer personId,
                                         @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize
                                         ) throws Exception {

        IPage<TaRecommender> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaRecommender> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("person_id", personId);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());
        queryWrapper.orderByDesc("create_date");
        return ResponseBean.success(iTaRecommenderService.list(queryWrapper));
    }

    /**
     * 保存对象
     * @param taRecommender 实体对象
     * @return
     */
    @RequestMapping(value="/wx/recommender",method= RequestMethod.POST)
    public ResponseBean taRecommenderAdd(@RequestBody TaRecommender taRecommender, HttpServletRequest request) throws Exception{
        TaPerson taPerson = getPerson(request);
        if (null == taPerson) {
            return ResponseBean.error("获取当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 推荐客户状态是未审核
        taRecommender.setStatus(StatusEnum.READY.getCode());
        // 设置当前推荐人
        taRecommender.setPersonId(taPerson.getPersonId());

        if (iTaRecommenderService.save(taRecommender)){
            return ResponseBean.success(taRecommender);
        }else {
            return ResponseBean.error("保存失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taRecommender/{id}", method= RequestMethod.DELETE)
    public ResponseBean taRecommenderDelete(@PathVariable Integer id) throws Exception{
        if(iTaRecommenderService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taRecommender 实体对象
     * @return
     */
    @RequestMapping(value="/admin/recommender/{id}",method= RequestMethod.PUT)
    public ResponseBean taRecommenderUpdate(@PathVariable Integer id,
                                        @RequestBody TaRecommender taRecommender) throws Exception{

        if (iTaRecommenderService.updateById(taRecommender)){
            return ResponseBean.success(iTaRecommenderService.getById(id));
        }else {
            return ResponseBean.error("更新失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taRecommender/{id}",method= RequestMethod.GET)
    public ResponseBean taRecommenderGet(@PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaRecommenderService.getById(id));
    }
}
