package com.yunzhi.liyuanhui.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.yunzhi.liyuanhui.common.StringUtils;
import com.yunzhi.liyuanhui.entity.TaActivity;
import com.yunzhi.liyuanhui.entity.TaActivityVote;
import com.yunzhi.liyuanhui.entity.TaActivityVoteItem;
import com.yunzhi.liyuanhui.entity.TaPerson;
import com.yunzhi.liyuanhui.enums.StatusEnum;
import com.yunzhi.liyuanhui.mapper.TaActivityVoteMapper;
import com.yunzhi.liyuanhui.service.ITaActivityService;
import com.yunzhi.liyuanhui.service.ITaActivityVoteItemService;
import com.yunzhi.liyuanhui.service.ITaActivityVoteService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 活动投票 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-08-08
 */
@Service
public class TaActivityVoteServiceImpl extends ServiceImpl<TaActivityVoteMapper, TaActivityVote> implements ITaActivityVoteService {

    @Autowired
    TaActivityVoteMapper taActivityVoteMapper;

    @Autowired
    ITaActivityService iTaActivityService;

    @Autowired
    ITaActivityVoteItemService iTaActivityVoteItemService;

    @Transactional
    @Override
    public boolean vote(TaActivity taActivity, TaActivityVoteItem taActivityVoteItem, TaPerson taPerson) throws Exception {
        if (null == taActivity || null == taActivityVoteItem || null == taPerson) {
            throw new Exception("参数错误");
        }

        if (StringUtils.isEmpty(taPerson.getPhone())) {
            throw new Exception("请先进行授权手机号操作");
        }

        if (!taActivity.getIsVote() || !StatusEnum.NORMAL.getCode().equals(taActivity.getStatus()) || !taActivity.getIsVote()) {
            throw new Exception("当前活动暂不支持投票");
        }

        if (!taActivity.getActivityId().equals(taActivityVoteItem.getActivityId())) {
            throw new Exception("投票内容与活动不符");
        }

        // 当前人员是否已经进行过投票
        QueryWrapper<TaActivityVote> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("activity_id", taActivity.getActivityId());
        queryWrapper.eq("item_id", taActivityVoteItem.getItemId());
        queryWrapper.eq("person_id", taPerson.getPersonId());
        int peronVoted = this.count(queryWrapper);
        if (peronVoted > 0) {
            throw new Exception("当前人员已经投过票");
        }

        // 投票
        TaActivityVote taActivityVote = new TaActivityVote();
        taActivityVote.setActivityId(taActivity.getActivityId());
        taActivityVote.setActivityName(taActivity.getName());
        taActivityVote.setActivityThumb(taActivity.getThumb());
        taActivityVote.setItemId(taActivityVoteItem.getItemId());
        taActivityVote.setItemName(taActivityVoteItem.getName());
        taActivityVote.setPersonId(taPerson.getPersonId());
        taActivityVote.setPersonName(StringUtils.ifNull(taPerson.getName(), taPerson.getNickname()));
        taActivityVote.setPersonAvatar(taPerson.getAvatar());
        taActivityVote.setPhone(taPerson.getPhone());

        if (!save(taActivityVote)) {
            throw new RuntimeException("投票失败, 请重试");
        }

        // 更新票数
        UpdateWrapper<TaActivityVoteItem> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("vote_num", taActivityVoteItem.getVoteNum() + 1);
        updateWrapper.eq("item_id", taActivityVoteItem.getItemId());
        updateWrapper.eq("vote_num", taActivityVoteItem.getVoteNum());
        boolean update = iTaActivityVoteItemService.update(updateWrapper);
        if (!update) {
            throw new RuntimeException("投票失败, 请重试");
        }

        // 更新投票总人数
        UpdateWrapper<TaActivity> updateWrapper2 = new UpdateWrapper<>();
        updateWrapper2.set("vote_num", taActivity.getVoteNum() + 1);
        updateWrapper2.eq("activity_id", taActivity.getActivityId());
        updateWrapper2.eq("vote_num", taActivity.getVoteNum());
        boolean update2 = iTaActivityService.update(updateWrapper2);
        if (!update2) {
            throw new RuntimeException("投票失败, 请重试");
        }

        return true;
    }

    @Override
    public boolean isVotedBy(Integer activityId, Integer personId) throws Exception {
        QueryWrapper<TaActivityVote> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("activity_id", activityId);
        queryWrapper.eq("person_id", personId);
        return this.count(queryWrapper) > 0;
    }

    @Override
    public TaActivityVote getBy(Integer activityId, TaPerson taPerson) throws Exception {
        QueryWrapper<TaActivityVote> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("activity_id", activityId);
        queryWrapper.eq("person_id", taPerson.getPersonId());
        return this.getOne(queryWrapper);
    }
}
