package com.yunzhi.liyuanhui.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.liyuanhui.common.StringUtils;
import com.yunzhi.liyuanhui.entity.TaPerson;
import com.yunzhi.liyuanhui.enums.StatusEnum;
import com.yunzhi.liyuanhui.mapper.TaPersonMapper;
import com.yunzhi.liyuanhui.service.ITaPersonService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 用户 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-08-08
 */
@Service
public class TaPersonServiceImpl extends ServiceImpl<TaPersonMapper, TaPerson> implements ITaPersonService {

    @Override
    public TaPerson getOrSavePerson(String openid) throws Exception {
        QueryWrapper<TaPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("openid", openid);
        TaPerson person = this.getOne(queryWrapper);

        // 如果找不到则新建一个
        if (null == person) {
            person = new TaPerson();
            person.setOpenid(openid);
            save(person);
            // 返回最新内容
            return this.getOne(queryWrapper);
        }

        return person;
    }

    @Override
    public TaPerson updatePersonPhone(Integer personId, String phone) throws Exception {
        TaPerson taPerson = this.getById(personId);
        if (null == taPerson) {
            throw new Exception("当前人员信息不存在");
        }

        taPerson.setPhone(phone);
        updateById(taPerson);
        return taPerson;
    }

    @Override
    public List<TaPerson> getListBy(String phone, String name) throws Exception {
        QueryWrapper<TaPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(phone), "phone", phone);
        queryWrapper.nested(!StringUtils.isEmpty(name), q -> q.like("name", name).or().like("nickname", name));
        queryWrapper.like(!StringUtils.isEmpty(name), "name", name);
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.nested(q -> q.and(qa -> qa.isNotNull("nickname").ne("nickname", ""))
                .or(qb -> qb.isNotNull("phone").ne("phone", "")));
        queryWrapper.orderByDesc("create_date");
        return this.list(queryWrapper);
    }
}
